﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyClock
{
    public partial class About : Form
    {
        public About()
        {
            InitializeComponent();
        }

        /// <Ver 1.16.2>Aboutダイアログで起動時表示のフラグ
        public static bool m_show_about;

        /// <Ver 1.14.1>Aboutダイアログ描画イベントハンドラ
        private void About_Load(object sender, EventArgs e)
        {
            // <Ver 1.14.1>起動時にAboutダイアログを表示するかどうか
            if (m_show_about)
                checkBox_about_show.CheckState = CheckState.Checked;
            else
                checkBox_about_show.CheckState = CheckState.Unchecked;

            // <Ver 1.16.9>[F1]キーから表示した時のみ[外観の設定]ボタンを表示する
            if (Display.m_crtl_available)
                button_about_ctrl.Visible = true;
            else
                button_about_ctrl.Visible = false;
            // <Ver 1.16.9>[外観の設定]ボタン表示フラグをfalseに戻す
            Display.m_crtl_available = false;

            // <Ver 1.14.2>バージョン情報表示 - アセンブリ情報を利用
            string msg = "";
            this.label_about_itle.Text = getAssemblyTitle();
            msg = getAssemblyProduct() + " Ver. " + getAssemblyVersion() + "\r\n";
            msg += getAssemblyCopyright() + "\r\n\r\n";
            textBox_about_prodctInfo.Text = msg;
            //<Ver 1.16.3>OKボタンにフォーカス
            button_about_OK.Select();

            // <Ver 1.16.3>Aboutダイアログを時計の周りに重ならないように表示する
            // <Ver 1.16.3>Aboutダイアログがあるディスプレイの高さと幅を取得
            int screenH = System.Windows.Forms.Screen.GetBounds(this).Height;
            int screenW = System.Windows.Forms.Screen.GetBounds(this).Width;

            // <Ver 1.16.3>MyClockフォームの左上の位置(left, top, width, height)を取得
            var left = this.Owner.Location.X;
            var top = this.Owner.Location.Y;
            var width = this.Owner.Width;
            var height = this.Owner.Height;

            // <Ver 1.16.3>Aboutダイアログのデフォルトの位置(Dislayフォームの右隣)
            int X = left + width;
            int Y = top;
            // <Ver 1.16.3>Aboutダイアログがスクリーンからはみ出す時、位置(X, Y)を更新
            if (X + this.Width > screenW)
                X = left - this.Width;
            if (Y + this.Height > screenH)
                Y = top + height - this.Height;

            // <Ver 1.16.3>親ウィンドウ (MyClock)の周りに移動させる
            this.Location = new Point(X, Y);
        }

        /// <Ver 1.14.2>アセンブリ情報の「タイトル」を返す
        public static string getAssemblyTitle()
        {
            System.Reflection.AssemblyTitleAttribute title =
                (System.Reflection.AssemblyTitleAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyTitleAttribute));

            return title.Title.ToString();
        }

        /// <Ver 1.14.2>アセンブリ情報の「製品」を返す - 他クラスから使うので public
        public static string getAssemblyProduct()
        {
            System.Reflection.AssemblyProductAttribute product =
                (System.Reflection.AssemblyProductAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyProductAttribute));

            return product.Product.ToString();
        }

        /// <Ver 1.14.2>アセンブリ情報の「バージョン」を返す
        public static string getAssemblyVersion()
        {
            //自分自身のAssembly情報を取得
            System.Reflection.Assembly assembly =
                System.Reflection.Assembly.GetExecutingAssembly();
            //バージョンの取得
            System.Version ver = assembly.GetName().Version;

            return ver.ToString();
        }

        /// <Ver 1.14.2>アセンブリ情報の「著作権」を返す
        private string getAssemblyCopyright()
        {
            System.Reflection.AssemblyCopyrightAttribute copyright =
                (System.Reflection.AssemblyCopyrightAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyCopyrightAttribute));

            return copyright.Copyright.ToString();
        }

        /// <Ver 1.16.2>チェックボックスのイベントハンドラ
        private void checkBox_about_show_CheckedChanged(object sender, EventArgs e)
        {
            switch (checkBox_about_show.CheckState)
            {
                case CheckState.Checked:
                    m_show_about = true;
                    break;
                case CheckState.Unchecked:
                case CheckState.Indeterminate:
                    m_show_about = false;
                    break;
            }
            button_about_OK.Select();
        }

        /// <Ver 1.16.9>外観変更ボタンのイベントハンドラ
        private void button_about_ctrl_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Yes;
            Dispose();
        }
    }
}
