﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyClock
{
    class Config
    {
        private string GetIniFilePath()
        {
            string fullpath = "";
            bool err;
            // MyClock.exe のパスを取得し、MyClock.ini のフルパスを得る
            fullpath = System.IO.Path.GetDirectoryName(
                Application.ExecutablePath) + "\\MyClock.ini";
            // MyClock.ini の存在を確認する
            err = !System.IO.File.Exists(fullpath);
            // MyClock.ini が存在しない時は空の MyClock.ini を作る
            if (err)
            {
                using (System.IO.FileStream hStream = System.IO.File.Create(fullpath))
                {
                    // 作成時に返される FileStream を利用して閉じる
                    if (hStream != null)
                    {
                        hStream.Close();
                    }
                }
            }
            return fullpath;
        }

        /// <summary>
        /// INIファイル (MyCLock.ini) に書き込む
        /// </summary>
        public void SetIniFile()
        {
            string inifile = GetIniFilePath();

            IniFile.SetIniString("＜＜注意＞＞", "***", "以下の設定は直接編集しないこと！***", inifile);
            IniFile.SetIniString("Assembly Info", "Title", About.getAssemblyTitle(), inifile);
            IniFile.SetIniString("Assembly Info", "Product", About.getAssemblyProduct(), inifile);
            IniFile.SetIniString("Assembly Info", "Verson", About.getAssemblyVersion(), inifile);
            IniFile.SetIniString("Application", "Alarm Time", Display.m_Alarm, inifile);
            IniFile.SetIniString("Application", "Alarm Canceled", Display.m_cancelAlarm ? "true" : "false", inifile);
            IniFile.SetIniString("Application", "Position.X", Display.m_main_pos.X.ToString(), inifile);
            IniFile.SetIniString("Application", "Position.Y", Display.m_main_pos.Y.ToString(), inifile);
            IniFile.SetIniString("Application", "isSize_normal", Display.m_size_normal ? "true" : "false", inifile);
            IniFile.SetIniString("Application", "Norm_alarm", Display.m_Norm_alarm.ToString(), inifile);
            IniFile.SetIniString("Application", "Icon_alarm", Display.m_Icon_alarm.ToString(), inifile);
            IniFile.SetIniString("Application", "doesShow_about", Display.m_show_about ? "true" : "false", inifile);
            IniFile.SetIniString("Application", "isStick", Display.m_stickDisp ? "true" : "false", inifile);
            IniFile.SetIniString("通常の外観", "背景色", Display.m_NormBackColor, inifile);
            IniFile.SetIniString("アラーム設定時の外観", "背景色", Display.m_SetAlarmBackColor, inifile);
            IniFile.SetIniString("通常の外観", "フォント.Name", Display.m_NormFont.Name.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.Size", Display.m_NormFont.Size.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.Style", Display.m_NormFont.Style.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.Unit", Display.m_NormFont.Unit.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.GdiCharSet", Display.m_NormFont.GdiCharSet.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "フォント.GdiVertialFont", Display.m_NormFont.GdiVerticalFont.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Name", Display.m_SetAlarmFont.Name.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Size", Display.m_SetAlarmFont.Size.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Style", Display.m_SetAlarmFont.Style.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Unit", Display.m_SetAlarmFont.Unit.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.GdiCharSet", Display.m_SetAlarmFont.GdiCharSet.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.GdiVerticalFont", Display.m_SetAlarmFont.GdiVerticalFont.ToString(), inifile);

            IniFile.SetIniString("通常の外観", "フォント.Color", Display.m_NormFontColor, inifile);
            IniFile.SetIniString("アラーム設定時の外観", "フォント.Color", Display.m_SetAlarmFontColor, inifile);
            IniFile.SetIniString("通常の外観", "サイズ.Width", Display.m_DispNormSize.Width.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "サイズ.Height", Display.m_DispNormSize.Height.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "サイズ.Width", Display.m_DispSetAlarmSize.Width.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "サイズ.Height", Display.m_DispSetAlarmSize.Height.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "不透明度", Display.m_NormOpacity.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "不透明度", Display.m_SetAlarmOpacity.ToString(), inifile);
            IniFile.SetIniString("通常の外観", "枠のスタイル", Display.m_NormBorder.ToString(), inifile);
            IniFile.SetIniString("アラーム設定時の外観", "枠のスタイル", Display.m_SetAlarmBorder.ToString(), inifile);
        }

        /// <summary>
        /// アプリケーション構成ファイル (MyClock.exe.config) に書き込む
        /// </summary>
        public void SetConfig()
        {
            //GetFormPos(); //DoubleClickイベントハンドラで、時計フォームの位置を取得しておく
            Properties.Settings.Default.alarm_Time = Display.m_Alarm;   // アラーム設定時刻
            Properties.Settings.Default.alarm_Canceled = Display.m_cancelAlarm; // アラーム解除設定
            Properties.Settings.Default.disp_Pos = Display.m_main_pos;   // 時計の位置
            Properties.Settings.Default.disp_Size_Normal = Display.m_size_normal;   // 時計のサイズ
            Properties.Settings.Default.disp_Norm_alarm = Display.m_Norm_alarm;   //前面表示⇒前面アラーム
            Properties.Settings.Default.disp_Icon_alarm = Display.m_Icon_alarm;   //アイコン⇒前面アラーム
            Properties.Settings.Default.show_About = Display.m_show_about;  //起動時メッセージ
            Properties.Settings.Default.norm_BackColor = Display.m_NormBackColor;   //通常背景色
            Properties.Settings.Default.setAlarm_BackColor = Display.m_SetAlarmBackColor;    //アラーム設定時背景色
            Properties.Settings.Default.norm_Font = Display.m_NormFont; //通常フォント
            Properties.Settings.Default.setAlarm_Font = Display.m_SetAlarmFont; //アラーム設定時フォント
            Properties.Settings.Default.norm_FontColor = Display.m_NormFontColor;   //通常フォント色
            Properties.Settings.Default.setAlarm_FontColor = Display.m_SetAlarmFontColor;    //アラーム設定時フォント色
            Properties.Settings.Default.disp_norm_size = Display.m_DispNormSize;    //通常の時計サイズ
            Properties.Settings.Default.disp_setalarm_size = Display.m_DispSetAlarmSize;    //アラーム設定時の時計サイズ
            Properties.Settings.Default.norm_Opacity = Display.m_NormOpacity;    //時計の不透明度
            Properties.Settings.Default.setAlarm_Opacity = Display.m_SetAlarmOpacity;    //時計の不透明度
            Properties.Settings.Default.norm_Border = Display.m_NormBorder; //枠のスタイル
            Properties.Settings.Default.setAlarm_Border = Display.m_SetAlarmBorder; //枠のスタイル
            Properties.Settings.Default.stickDisp = Display.m_stickDisp; //張り付きフラグ
            Properties.Settings.Default.Save();
        }

        public void GetIniFile()
        {
            string inifile = GetIniFilePath(), s = "";

            s = IniFile.GetIniString("Application", "Alarm Time", inifile);
            if (s != "") { Display.m_Alarm = s; }
            else { Display.m_Alarm = "00:00:00"; }

            s = IniFile.GetIniString("Application", "Alarm Canceled", inifile);
            if (s == "true") { Display.m_cancelAlarm = true; }
            else { Display.m_cancelAlarm = false; }

            s = IniFile.GetIniString("Application", "Position.X", inifile);
            if (s != "") { Display.m_main_pos.X = int.Parse(s); }
            else { Display.m_main_pos.X = 100; }

            s = IniFile.GetIniString("Application", "Position.Y", inifile);
            if (s != "") { Display.m_main_pos.Y = int.Parse(s); }
            else { Display.m_main_pos.Y = 70; }

            s = IniFile.GetIniString("Application", "isSize_normal", inifile);
            if (s == "false") { Display.m_size_normal = false; }
            else { Display.m_size_normal = true; }

            s = IniFile.GetIniString("Application", "Norm_alarm", inifile);
            if (s != "") { Display.m_Norm_alarm = int.Parse(s); }
            else { Display.m_Norm_alarm = 1; }

            s = IniFile.GetIniString("Application", "Icon_alarm", inifile);
            if (s != "") { Display.m_Icon_alarm = int.Parse(s); }
            else { Display.m_Icon_alarm = 2; }

            s = IniFile.GetIniString("Application", "doesShow_about", inifile);
            if (s == "false") { Display.m_show_about = false; }
            else { Display.m_show_about = true; }

            s = IniFile.GetIniString("Application", "isStick", inifile);
            if (s == "true") { Display.m_stickDisp = true; }
            else { Display.m_stickDisp = false; }

            s = IniFile.GetIniString("通常の外観", "背景色", inifile);
            if (s != "") { Display.m_NormBackColor = s; }
            else { Display.m_NormBackColor = "Info"; }

            s = IniFile.GetIniString("アラーム設定時の外観", "背景色", inifile);
            if (s != "") { Display.m_SetAlarmBackColor = s; }
            else { Display.m_SetAlarmBackColor = "ActiveCaption"; }

            s = IniFile.GetIniString("通常の外観", "フォント.Name", inifile);
            string name = s;
            if (name == "") { name = "MS UI Gothic"; }
            string size = IniFile.GetIniString("通常の外観", "フォント.Size", inifile);
            if (size == "") { size = "36F"; }
            string style = IniFile.GetIniString("通常の外観", "フォント.Style", inifile);
            if (style == "") { style = "Regular"; }
            string unit = IniFile.GetIniString("通常の外観", "フォント.Unit", inifile);
            if (unit == "") { unit = "Point"; }
            string gcs = IniFile.GetIniString("通常の外観", "フォント.GdiCharSet", inifile);
            if (gcs == "") { gcs = "1"; }
            string gvf = IniFile.GetIniString("通常の外観", "フォント.GdiVertialFont", inifile);
            if (gvf == "") { gvf = "False"; }
            if (s != "")
            {
                Display.m_NormFont = new Font(
                    name,
                    float.Parse(size),
                    (FontStyle)Enum.Parse(typeof(FontStyle), style),
                    (GraphicsUnit)Enum.Parse(typeof(GraphicsUnit), unit),
                    byte.Parse(gcs),
                    bool.Parse(gvf));
            }
            else
            {
                Display.m_NormFont = new Font("MS UI Gothic", 36F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(128)));
            }

            s = IniFile.GetIniString("アラーム設定時の外観", "フォント.Name", inifile);
            name = s;
            if (name == "") { name = "MS UI Gothic"; }
            size = IniFile.GetIniString("アラーム設定時の外観", "フォント.Size", inifile);
            if (size == "") { size = "36F"; }
            style = IniFile.GetIniString("アラーム設定時の外観", "フォント.Style", inifile);
            if (style == "") { style = "Regular"; }
            unit = IniFile.GetIniString("アラーム設定時の外観", "フォント.Unit", inifile);
            if (unit == "") { unit = "Point"; }
            gcs = IniFile.GetIniString("アラーム設定時の外観", "フォント.GdiCharSet", inifile);
            if (gcs == "") { gcs = "1"; }
            gvf = IniFile.GetIniString("アラーム設定時の外観", "フォント.GdiVertialFont", inifile);
            if (gvf == "") { gvf = "False"; }
            if (s != "")
            {
                Display.m_SetAlarmFont = new Font(
                    name,
                    float.Parse(size),
                    (FontStyle)Enum.Parse(typeof(FontStyle), style),
                    (GraphicsUnit)Enum.Parse(typeof(GraphicsUnit), unit),
                    byte.Parse(gcs),
                    bool.Parse(gvf));
            }
            else
            {
                Display.m_SetAlarmFont = new Font("MS UI Gothic", 36F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(128)));
            }

            s = IniFile.GetIniString("通常の外観", "フォント.Color", inifile);
            if (s != "") { Display.m_NormFontColor = s; }
            else { Display.m_NormFontColor = "ControlText"; }

            s = IniFile.GetIniString("アラーム設定時の外観", "フォント.Color", inifile);
            if (s != "") { Display.m_SetAlarmFontColor = s; }
            else { Display.m_SetAlarmFontColor = "ControlText"; }

            s = IniFile.GetIniString("通常の外観", "サイズ.Width", inifile);
            if (s != "") { Display.m_DispNormSize.Width = int.Parse(s); }
            else { Display.m_DispNormSize.Width = 191; }

            s = IniFile.GetIniString("通常の外観", "サイズ.Height", inifile);
            if (s != "") { Display.m_DispNormSize.Height = int.Parse(s); }
            else { Display.m_DispNormSize.Height = 46; }

            s = IniFile.GetIniString("アラーム設定時の外観", "サイズ.Width", inifile);
            if (s != "") { Display.m_DispSetAlarmSize.Width = int.Parse(s); }
            else { Display.m_DispSetAlarmSize.Width = 191; }

            s = IniFile.GetIniString("アラーム設定時の外観", "サイズ.Height", inifile);
            if (s != "") { Display.m_DispSetAlarmSize.Height = int.Parse(s); }
            else { Display.m_DispSetAlarmSize.Height = 46; }

            s = IniFile.GetIniString("通常の外観", "不透明度", inifile);
            if (s != "") { Display.m_NormOpacity = double.Parse(s); }
            else { Display.m_NormOpacity = 0.5D; }

            s = IniFile.GetIniString("アラーム設定時の外観", "不透明度", inifile);
            if (s != "") { Display.m_SetAlarmOpacity = double.Parse(s); }
            else { Display.m_SetAlarmOpacity = 0.5D; }

            s = IniFile.GetIniString("通常の外観", "枠のスタイル", inifile);
            if (s != "") { Display.m_NormBorder = s; }
            else { Display.m_NormBorder = "None"; }

            s = IniFile.GetIniString("アラーム設定時の外観", "枠のスタイル", inifile);
            if (s != "") { Display.m_SetAlarmBorder = s; }
            else { Display.m_SetAlarmBorder = "None"; }
        }

        /// <summary>
        /// アプリケーション構成ファイル (MyClock.exe.config) から取得する
        /// </summary>
        public void GetConfig()
        {
            //前のバージョンの設定を読み込み、新しいバージョンの設定とする
            Properties.Settings.Default.Upgrade();
            Display.m_Alarm = Properties.Settings.Default.alarm_Time;   // アラーム設定時刻
            Display.m_cancelAlarm = Properties.Settings.Default.alarm_Canceled; //アラーム解除設定
            Display.m_main_pos = Properties.Settings.Default.disp_Pos;    //時計表示位置
            Display.m_size_normal = Properties.Settings.Default.disp_Size_Normal;   //時計サイズ
            Display.m_Norm_alarm = Properties.Settings.Default.disp_Norm_alarm;   //前面表示⇒前面アラーム
            Display.m_Icon_alarm = Properties.Settings.Default.disp_Icon_alarm;   //アイコン⇒前面アラーム
            Display.m_show_about = Properties.Settings.Default.show_About;  //起動時メッセージ
            Display.m_NormBackColor = Properties.Settings.Default.norm_BackColor;   //通常背景色
            Display.m_SetAlarmBackColor = Properties.Settings.Default.setAlarm_BackColor;    //アラーム設定時背景色
            Display.m_NormFont = Properties.Settings.Default.norm_Font; //通常フォント
            Display.m_SetAlarmFont = Properties.Settings.Default.setAlarm_Font; //アラーム設定時フォント
            Display.m_NormFontColor = Properties.Settings.Default.norm_FontColor;   //通常フォント色
            Display.m_SetAlarmFontColor = Properties.Settings.Default.setAlarm_FontColor;    //アラーム設定時フォント色
            Display.m_DispNormSize = Properties.Settings.Default.disp_norm_size;
            Display.m_DispSetAlarmSize = Properties.Settings.Default.disp_setalarm_size;
            Display.m_NormOpacity = Properties.Settings.Default.norm_Opacity;    //時計の不透明度
            Display.m_SetAlarmOpacity = Properties.Settings.Default.setAlarm_Opacity;    //時計の不透明度
            Display.m_NormBorder = Properties.Settings.Default.norm_Border; //枠のスタイル
            Display.m_SetAlarmBorder = Properties.Settings.Default.setAlarm_Border; //枠のスタイル
            Display.m_stickDisp = Properties.Settings.Default.stickDisp;    //張り付きフラグ
        }
    }
}
