﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyClock
{
    public partial class Ctrl : Form
    {
        public Ctrl()
        {
            InitializeComponent();
            // <Ver 1.17>アラーム時刻を保存 - [通常時] / [アラーム設定時] を強制切替するため
            m_savedAlarmTime = Display.m_Alarm;
            // <Ver 1.17>[外観の設定]起動時に[アラームの設定]の起動を許可しない
            Display.m_allowAlarmDisp = false;
        }

        // <Ver 1.17.0>"通常の外観"かどうかをこのフラグのみで制御する
        public static bool status = Display.m_normDisp;
        
        //<Ver 1.17.0>[通常の外観]ボタンのフラグ
        public static bool m_ctrlNormChecked;

        //<Ver 1.17.0>アラーム時刻の一時記憶用変数
        public static string m_savedAlarmTime;

        // <Ver 1.17.3>外観切替ボタンのテキスト
        private static string text_Norm = "通常の外観";
        private static string text_SetAlarm = "アラーム設定時の外観";

        /// <Ver 1.16.9>時計の外観プロパティのクラス
        [DefaultProperty("DispOpacity")]
        class DispSetting
        {
            [Category("表示")]
            [DisplayName("背景色")]
            [Description("時計の背景色を設定します.\n(既定の設定はありません)")]
            //[DefaultValue(typeof(Color), "Info")] // <Ver 1.17.3>既定値を設定しない
            public Color BackgroundColor { get; set; }

            [Category("表示")]
            [DisplayName("フォント")]
            [Description("時刻表示のフォントを設定します.")]
            [DefaultValue(typeof(Font), "MS UI Gothic, 36pt")]
            public Font Font { get; set; }

            [Category("表示")]
            [DisplayName("フォント色")]
            [Description("時刻表示のフォント色を設定します.")]
            [DefaultValue(typeof(Color), "ControlText")]
            public Color FontColor { get; set; }

            [Category("配置")]
            [DisplayName("サイズ")]
            [Description("時計のサイズを設定します.")]
            [DefaultValue(typeof(Size), "191, 46")]
            public Size DispSize { get; set; }

            [Category("ウィンドウスタイル")]
            [DisplayName("不透明度")]
            [Description("時計の不透明度を 0.1 ～ 1 の間で設定します.")]
            [DefaultValue(typeof(double), "0.5")]
            public double DispOpacity { get; set; }

            [Category("表示")]
            [DisplayName("枠のスタイル")]
            [Description("時計の枠の外観を設定します.")]
            [DefaultValue(typeof(BorderStyle), "None")]
            public BorderStyle DispBorder { get; set; }
        }
        DispSetting c = new DispSetting();  // プロパティのインスタンス生成

        /// <Ver 1.16.9>Ctrlダイアログ描画のイベントハンドラ
        private void Ctrl_Load(object sender, EventArgs e)
        {
            // <Ver 1.17.3>プロパティグリッドにプロパティを設定する
            setProperties();

            // <Ver 1.17.3>[既定の設定に戻す]ボタン Enable = true / false の制御
            ControlDfaultButton();

            // <Ver 1.16.9>プロパティグリッドの外観制御
            // [通常時] か [アラーム設定時] かの違いが視覚的に分かりやすく変化させる
            if (Display.m_normDisp)
            {
                property_Display.LineColor = SystemColors.InactiveBorder;
                button_ctrl_Norm.ForeColor = Color.Black;
                button_ctrl_Norm.Text = text_Norm;
                m_ctrlNormChecked = true;
            }
            else
            {
                property_Display.LineColor = Color.Snow;
                button_ctrl_Norm.ForeColor = Color.Red;
                button_ctrl_Norm.Text = text_SetAlarm;
                m_ctrlNormChecked = false;
            }

            // <Ver 1.16.9>外観設定ダイアログを時計の周りに重ならないように表示する
            // <Ver 1.17.3>メンバー呼び出しに変更
            MoveCtrlForm();

            // <Ver 1.17.3b>[張り付かない]ボタンテキストの初期化
            if (Display.m_stickDisp)
            {
                button_Stick.Text = "貼り付く";
                Display.DispInstance.DispStickTimer(true);
                Display.m_stickDisp = true;
            }
            else
            {
                button_Stick.Text = "貼り付かない";
                Display.DispInstance.DispStickTimer(false);
                Display.m_stickDisp = false;
            }

        }

        /// <Ver 1.16.9>プロパティの値を変更する時の処理
        public void setChangedPropertyValues()
        {
            string itemName = property_Display.SelectedGridItem.Label.ToString();
            switch (itemName)
            {
                case "背景色":
                    if ((Display.DispInstance.DispBackColor = this.c.BackgroundColor) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            Display.m_NormBackColor = this.c.BackgroundColor.Name.ToString();
                        }
                        else
                        {
                            Display.m_SetAlarmBackColor = this.c.BackgroundColor.Name.ToString();
                        }
                    }
                    break;

                case "フォント":
                case "Name":
                case "Size":
                case "Unit":
                case "Bold":
                case "GdiHarSet":
                case "GdiVerticalFont":
                case "Italic":
                case "Strikeout":
                case "Underline":
                    if ((Display.DispInstance.DispFont = this.c.Font) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            Display.m_NormFont = this.c.Font;
                        }
                        else
                        {
                            Display.m_SetAlarmFont = this.c.Font;
                        }
                    }
                    break;

                case "フォント色":
                    if ((Display.DispInstance.DispForeColor = this.c.FontColor) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            Display.m_NormFontColor = this.c.FontColor.Name.ToString();
                        }
                        else
                        {
                            Display.m_SetAlarmFontColor = this.c.FontColor.Name.ToString();
                        }
                    }
                    break;

                case "サイズ":
                case "Width":
                case "Height":
                    if ((Display.DispInstance.DispSize = this.c.DispSize) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            Display.m_DispNormSize = this.c.DispSize;
                        }
                        else
                        {
                            Display.m_DispSetAlarmSize = this.c.DispSize;
                        }
                    }
                    break;

                case "不透明度":
                    Display.DispInstance.DispOpacity = this.c.DispOpacity;
                    {
                        double d = this.c.DispOpacity;
                        if (d < 0.1D)
                            d = 0.1D;
                        else if (d > 1D)
                            d = 1.0D;
                        property_Display.SelectedGridItem.PropertyDescriptor.SetValue(
                            property_Display.SelectedObject, d);
                        if (Display.m_normDisp)
                        {
                            Display.m_NormOpacity = d;
                        }
                        else
                        {
                            Display.m_SetAlarmOpacity = d;
                        }
                    }
                    break;

                case "枠のスタイル":
                    Display.DispInstance.DispBorder = this.c.DispBorder;
                    {
                        if (Display.m_normDisp)
                        {
                            Display.m_NormBorder = this.c.DispBorder.ToString();
                        }
                        else
                        {
                            Display.m_SetAlarmBorder = this.c.DispBorder.ToString();
                        }
                    }
                    break;
            }
        }

        // <Ver 1.17.3>[既定の設定に戻す]ボタン Enable = true / false を制御する
        private void ControlDfaultButton()
        {
            if (property_Display.SelectedGridItem.GridItemType == GridItemType.Property)
            {
                //選択中のアイテムがキャンセル可能なら[既定の設定]ボタンを使えるようにする
                button_setDefalt.Enabled =
                    property_Display.SelectedGridItem.PropertyDescriptor.CanResetValue(
                        property_Display.SelectedObject);
            }
            else
            {
                //キャンセル可能でないなら[既定の設定]ボタンをグレーアウトする
                button_setDefalt.Enabled = false;
            }
        }

        /// <Ver 1.16.9>プロパティの値を変更された時のイベントハンドラ
        private void property_Display_PropertyValueChanged(
                            object s, PropertyValueChangedEventArgs e)
        {
            ControlDfaultButton();
            setChangedPropertyValues();
        }

        /// <Ver 1.16.9>プロパティ項目の選択が変更された時のイベントハンドラ
        private void property_Display_SelectedGridItemChanged(
                object sender, SelectedGridItemChangedEventArgs e)
        {
            ControlDfaultButton();
        }

        /// <Ver 1.16.9>プロパティを既定の設置柄に戻す
        private void setDefaultPropertyValues()
        {
            string itemName = property_Display.SelectedGridItem.Label.ToString();
            //MessageBox.Show(itemName);
            switch (itemName)
            {
                case "背景色":
                    if ((Display.DispInstance.DispBackColor = this.c.BackgroundColor) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            if (Display.m_NormBackColor != null)
                                Display.m_NormBackColor = "Info";
                        }
                        else
                        {
                            if (Display.m_SetAlarmBackColor != null)
                            {
                                Display.m_SetAlarmBackColor = "ActiveCaption";
                            }
                        }
                    }
                    break;

                case "フォント":
                    if ((Display.DispInstance.DispFont = this.c.Font) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            if (Display.m_NormFont != null)
                                Display.m_NormFont =
                                    new Font("MS UI Gothic", 36F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(128)));
                        }
                        else
                        {
                            if (Display.m_SetAlarmFont != null)
                                Display.m_SetAlarmFont =
                                    new Font("MS UI Gothic", 36F, FontStyle.Regular, GraphicsUnit.Point, ((byte)(128)));
                        }
                    }
                    break;

                case "フォント色":
                    if ((Display.DispInstance.DispForeColor = this.c.FontColor) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            if (Display.m_NormFontColor != null)
                                Display.m_NormFontColor = "ControlText";
                        }
                        else
                        {
                            if (Display.m_SetAlarmFontColor != null)
                                Display.m_SetAlarmFontColor = "ControlText";
                        }
                    }
                    break;

                case "サイズ":
                    if ((Display.DispInstance.DispSize = this.c.DispSize) != null)
                    {
                        if (Display.m_normDisp)
                        {
                            if (Display.m_DispNormSize != null)
                            {
                                Display.m_DispNormSize.Width = 191;
                                Display.m_DispNormSize.Height = 46;
                            }
                        }
                        else
                        {
                            if (Display.m_DispNormSize != null)
                            {
                                Display.m_DispSetAlarmSize.Width = 191;
                                Display.m_DispSetAlarmSize.Height = 46;
                            }
                        }
                    }
                    break;

                case "不透明度":
                    if ((Display.DispInstance.DispOpacity = this.c.DispOpacity) > 0)
                    {
                        if (Display.m_normDisp)
                        {
                            Display.m_NormOpacity = 0.5D;
                        }
                        else
                        {
                            Display.m_SetAlarmOpacity = 0.5D;
                        }
                    }
                    break;
                case "枠のスタイル":
                    Display.DispInstance.DispBorder = this.c.DispBorder; //<Ver 1.17.3>修正
                    {
                        if (Display.m_normDisp)
                        {
                            if (Display.m_NormBorder != null)
                                Display.m_NormBorder = "None";
                        }
                        else
                        {
                            if (Display.m_SetAlarmBorder != null)
                                Display.m_SetAlarmBorder = "None";
                        }
                    }
                    break;
            }
            button_ctrl_Norm.Select();
        }

        /// <Ver 1.16.9>[閉じる]ボタンクリックのイベントハンドラ
        private void button_ctrl_OK_Click(object sender, EventArgs e)
        {
            Display.m_Alarm = m_savedAlarmTime;
            Display.m_allowAlarmDisp = true;    // アラーム設定画面の起動を許可
            Dispose();
        }

        /// <Ver 1.16.9>[既定値に戻す]ボタンクリックのイベントハンドラ
        private void button_setDefalt_Click(object sender, EventArgs e)
        {
            property_Display.ResetSelectedProperty();
            property_Display.Refresh();
            button_ctrl_Norm.Select();
            setDefaultPropertyValues();
        }

        // <Ver 1.17.3>プロパティグリッドにプロパティを設定する
        private void setProperties()
        {
            property_Display.SelectedObject = this.c;
            this.c.BackgroundColor = Display.DispInstance.DispBackColor;
            this.c.Font = Display.DispInstance.DispFont;
            this.c.FontColor = Display.DispInstance.DispForeColor;
            this.c.DispSize = Display.DispInstance.DispSize;
            this.c.DispOpacity = Display.DispInstance.DispOpacity;
            this.c.DispBorder = Display.DispInstance.DispBorder;
        }

        /// <Ver 1.16.9>プロパティのオブジェクトを入れ替える
        private void swapCtrl()
        {
            status = Display.m_normDisp; // statusフラグをリセットする
            Display.m_resumedDisp = false;  // アラーム発動からの復帰フラグを下げる
            if (Display.m_normDisp)
            {
                // <Ver 1.17.3>プロパティグリッドにプロパティを設定する
                setProperties();
                button_ctrl_Norm.ForeColor = Color.Black;
                button_ctrl_Norm.Text = text_Norm;
                property_Display.LineColor = SystemColors.InactiveBorder;
            }
            else
            {
                // <Ver 1.17.3>プロパティグリッドにプロパティを設定する
                setProperties();
                button_ctrl_Norm.ForeColor = Color.Red;
                button_ctrl_Norm.Text = text_SetAlarm;
                property_Display.LineColor = Color.Snow;
            }
            // <Ver 1.17.3>[既定の設定に戻す]ボタン Enable = true / false の制御
            ControlDfaultButton();
        }

        /// <Ver 1.17.3>外観設定ダイアログを時計の周りに重ならないように表示する
        private void MoveCtrlForm()
        {
            // 外観設定ダイアログがあるディスプレイの高さと幅を取得
            int screenH = System.Windows.Forms.Screen.GetBounds(this).Height;
            int screenW = System.Windows.Forms.Screen.GetBounds(this).Width;
            // MyClockフォームの左上の位置(left, top, width, height)を取得
            var left = this.Owner.Location.X;
            var top = this.Owner.Location.Y;
            var width = this.Owner.Width;
            var height = this.Owner.Height;
            // 外観設定ダイアログのデフォルトの位置(Dislayフォームの右隣)
            int X = left + width;
            int Y = top;
            // 外観設定ダイアログがスクリーンからはみ出す時、位置(X, Y)を更新
            if (X + this.Width > screenW)
                X = left - this.Width;
            if (Y + this.Height > screenH)
                Y = top + height - this.Height;

            // 親ウィンドウ (MyClock)の周りに移動させる
            this.Location = new Point(X, Y);
        }

        /// <Ver 1.16.9>定期的に Display.m_normDisp フラグをチェックして
        /// 変化したらプロパティオブジェクトを入れ替える
        private void timer_checkDisp_Tick(object sender, EventArgs e)
        {
            // sattus フラグ = 外観変化フラグが立てばプロパティオブジェクトを入れ替える
            if (status != Display.m_normDisp)
                swapCtrl();

            // アラーム発動からの復帰フラグが立てばプロパティオブジェクトを入れ替える
            if (Display.m_resumedDisp)
                swapCtrl();

            // <Ver 1.17.3>Display.m_alarm_modeがtrue時(アラーム発動時)にフォームを再配置
            // <Ver 1.17.3>プロパティ編集時にアラーム発動を受け付けるように仕様変更
            if (Display.m_alarm_mode)
                MoveCtrlForm();
        }

        /// <Ver 1.17.0>強制的に [通常表示] / [アラーム設定時表示] を切り替える
        private void button_ctrl_Norm_Click(object sender, EventArgs e)
        {
            if (m_ctrlNormChecked)
            {
                // <Ver 1.17.2>この２行のコメントアウトにより、早いボタンクリックでのプロパティ表示の混乱を抑制
                //button_ctrl_Norm.ForeColor = Color.Red;
                //button_ctrl_Norm.Text = "アラーム設定時の外観";
                Display.m_Alarm = "23:59:59";
                m_ctrlNormChecked = false;
            }
            else
            {
                // <Ver 1.17.2>この２行のコメントアウトにより、早いボタンクリックでのプロパティ表示の混乱を抑制
                //button_ctrl_Norm.ForeColor = Color.Black;
                //button_ctrl_Norm.Text = "通常の外観";
                Display.m_Alarm = "00:00:00";
                m_ctrlNormChecked = true;
            }
            button_ctrl_Norm.Select();
        }

        /// <Ver 1.70> Ctrl フォームが破棄された時のイベントハンドラ
        private void Ctrl_FormClosed(object sender, FormClosedEventArgs e)
        {
            Display.m_Alarm = m_savedAlarmTime;
            Display.m_allowAlarmDisp = true;    // アラーム設定画面の起動を許可
        }

        // <Ver 1.17.3.b>[張り付く]ボタンクリックのイベントハンドラ
        private void button_Stic_Click(object sender, EventArgs e)
        {
            if (Display.m_stickDisp)
            {
                button_Stick.Text = "貼り付かない";
                Display.DispInstance.DispStickTimer(false);
                Display.m_stickDisp = false;
            }
            else
            {
                button_Stick.Text = "貼り付く";
                Display.DispInstance.DispStickTimer(true);
                Display.m_stickDisp = true;
            }
            button_ctrl_Norm.Select();
        }
    }
}
