﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks; // .NET Franework 3.5 では使えないのでコメントアウト
using System.Windows.Forms;
using System.Runtime.InteropServices;   // DllImportのために追加
using System.Diagnostics; // ウィンドウプロセス取得のために追加

namespace MyClock
{
    public partial class Display : Form
    {
        public Display()
        {
            InitializeComponent();

            // <ver 1.12>設定ファイル (MyClock.exe.config) からの読み込み
            // <Ver 1.16.7> 設定ファイル入出力は、クラス Config にまとめた
            Config Conf = new Config();
            Conf.GetConfig();   // <Ver 1.17.3d> MyClock.exe.config を使う最後のバージョン 
            Conf.GetIniFile(); // <Ver 1.17.3d> これ以降は INI ファイルを設定ファイルとする

            // <Ver 1.16.4>起動時の構成変更
            ChangeConf dlg = new ChangeConf();
            var result = dlg.ShowDialog(this);
        }

        // <Ver 1.10>マウス位置を記憶する変数
        private Point mousePoint;

        // <Ver 1.10>アラーム時刻の格納変数 (class Config でファイル入出力)
        public static string m_Alarm;

        // <Ver 1.11>アラーム解除フラグの格納変数 (class Config でファイル入出力)
        public static bool m_cancelAlarm = false;

        //<Ver1.12>時計の位置の格納変数 (class Config でファイル入出力)
        //<Ver 1.16.2>Pointクラスを使って書き換え
        public static Point m_main_pos;

        //<Ver1.13>設定ファイルに保存する時計サイズの格納変数 (class Config でファイル入出力)
        public static bool m_size_normal;

        // <Ver 1.14>アラーム発動中フラグ
        // <Ver 1.17.3>[外観の設定] Ctrl クラスからモニタするためにpublic static に変更
        public static bool m_alarm_mode = false;

        // <Ver1.14>アラーム発動前の時計サイズ
        private bool m_display_norm;

        // <ver 1.14>アラーム画面の設定 (class Config でファイル入出力)
        public static int m_Norm_alarm;
        public static int m_Icon_alarm;

        // <Ver 1.15>アラーム発動直前の時計の位置
        // <Ver 1.16.2>Pointクラスを使って書き換え
        private Point m_temp_main_pos;

        // <Ver 1.16.2>Aboutダイアログで起動時表示のフラグ (class Config でファイル入出力)
        public static bool m_show_about = true;

        // <Ver 1.16.9>[外観の設定]ボタン表示フラグ
        public static bool m_crtl_available;

        // <Ver 1.16.9>背景色の設定 (class Config でファイル入出力)
        public static string m_NormBackColor;
        public static string m_SetAlarmBackColor;

        // <Ver 1.16.9>フォントの設定 (class Config でファイル入出力)
        public static Font m_NormFont;
        public static Font m_SetAlarmFont;

        // <Ver 1.16.9>フォント色の設定 (class Config でファイル入出力)
        public static string m_NormFontColor;
        public static string m_SetAlarmFontColor;

        // <Ver 1.16.9>サイズの設定 (class Config でファイル入出力)
        public static Size m_DispNormSize;
        public static Size m_DispSetAlarmSize;

        // <Ver 1.16.9>不透明度の設定 (class Config でファイル入出力)
        public static double m_NormOpacity;
        public static double m_SetAlarmOpacity;

        // <Ver 1.16.9>時計枠のスタイル (class Config でファイル入出力)
        public static string m_NormBorder;
        public static string m_SetAlarmBorder;

        // <Ver 1.16.9>通常/アラーム設定時の切替フラグ
        public static bool m_normDisp;

        // <Ver 1.16.9>アラーム発動から外観を元に戻すフラグ
        public static bool m_resumedDisp = false;

        // <Ver 1.17>[外観の設定]起動時に[アラム設定]起動を許すフラグ
        public static bool m_allowAlarmDisp = true;

        // <Ver 1.17.3b>キャプションバーに張り付く機能のフラグ (class Config でファイル入出力)
        public static bool m_stickDisp = false;

        // <Ver 1.17.3b>アラーム発動時に貼り付くフラグを一時保管する変数
        private bool m_temp_stickDisp;


        ///  Form描画のメッセージハンドラ
        private void Display_Load(object sender, EventArgs e)
        {
            DisplayTime();  // <Ver 1.00>起動時タイマー動作までのの時刻表示
            this.TopMost = true;    // <Ver 1.11>フォーム描画時に再度前面表示を指定

            // <Ver 1.13>前回終了時に最小化されていた時は最小化する
            if (!m_size_normal)
            {
                this.WindowState = FormWindowState.Minimized;
                m_size_normal = true;
            }

            // <Ver 1.12>終了時の時計の位置情報
            //this.Location = new Point(m_main_left, m_main_top);
            this.Location = m_main_pos;

            // <Ver 1.16.1>起動時にAboutダイアログを表示する
            if (m_show_about)
                showAbout();

            // <Ver 1.16.9>Displayクラスを静的インスタンス DispInstanceに代入する 
            Display.DispInstance = this;

            // <Ver 1.17.3b>貼り付くフラグがって居ると貼り付ける
            if (m_stickDisp)
            {
                DispStickTimer(true);
                GetActiveWindow();
            }
        }

        /// <Ver 1.13>時計フォームの位置を取得する
        /// <Ver 1.15.1>参照渡しで書き換えて共通化
        /// <Ver 1.16.2>Pointクラスを使って書き換える
        private void GetFormPos(ref Point pos)
        {
            pos = new Point(this.Left, this.Top);
        }

        /// <Ver 1.00>Timerのメッセージハンドラ - 現在の時刻とWindowStateを１秒ごとに取得
        private void secTimer_Tick(object sender, EventArgs e)
        {
            DisplayTime();
            if (this.WindowState == FormWindowState.Normal)
            {
                m_size_normal = true;
            }
            else if (this.WindowState == FormWindowState.Minimized)
            {
                m_size_normal = false;
            }
        }

        /// <Ver 1.00>現在時刻の表示
        private void DisplayTime()
        {
            string time = DateTime.Now.ToString("HH:mm:ss");
            currentTime.Text = time;
            // <Ver 1.16.7>最小化時にアイコン状態で現在時刻を表示する
            if (WindowState == FormWindowState.Minimized)
            {
                this.Text = "MyClock - " + time;
            }

            // <Ver 1.10>アラーム設定時間になったら時計の外観を変えて点滅させる
            // <Ver 1.17.3>プロパティ編集時にアラーム発動を受け付けるように仕様変更
            if (time ==m_Alarm || time == Ctrl.m_savedAlarmTime)
            {
                SetDisp(DISP_NORM);  // 通常の外観にする
                InvokeAlarm();  //<Ver 1.15.1>アラーム発動部分を別関数に丸ごとコピー
            }
            else if (DateTime.Parse(time) < DateTime.Parse(m_Alarm))
            {
                // <Ver 1.16.5>アラーム設定されている時 (現在時刻＜アラーム時刻) 
                if (!m_cancelAlarm)
                {
                    // アラーム設定時の外観にする
                    SetDisp(DISP_SETALARM);
                }
                else
                {
                    // 通常の外観にする
                    SetDisp(DISP_NORM);
                }
            }
            else if (DateTime.Parse(time) > DateTime.Parse(m_Alarm))
            {
                // <Ver 1.16.5>アラーム設定されていない時 (現在時刻＞アラーム時刻) は通常の外観にする
                SetDisp(DISP_NORM);
            }
        }
        
        /// <Ver 1.16.9>定数宣言
        public const bool DISP_NORM = true;
        public const bool DISP_SETALARM = false;

        /// <Ver 1.16.9>外観設定を反映させる
        private void SetDisp(bool flag)
        {
            switch (flag)
            {
                case true:
                    //this.currentTime.BackColor = SystemColors.Info;   //既定の背景
                    this.currentTime.BackColor = Color.FromName(m_NormBackColor);
                    this.currentTime.Font = m_NormFont;
                    this.currentTime.ForeColor = Color.FromName(m_NormFontColor);
                    this.ClientSize = m_DispNormSize;
                    if (!m_alarm_mode)  // アラーム発動中は ChangeDisplay()の設定に従う
                        this.Opacity = m_NormOpacity;
                    this.currentTime.BorderStyle =  (BorderStyle)Enum.Parse(typeof(FormBorderStyle), m_NormBorder);
                    m_normDisp = true;
                    break;
                case false:
                    //this.currentTime.BackColor = SystemColors.ActiveCaption;   //規定の背景
                    this.currentTime.BackColor = Color.FromName(m_SetAlarmBackColor);
                    this.currentTime.Font = m_SetAlarmFont;
                    this.currentTime.ForeColor = Color.FromName(m_SetAlarmFontColor);
                    this.ClientSize = m_DispSetAlarmSize;
                    this.Opacity = m_SetAlarmOpacity;
                    this.currentTime.BorderStyle = (BorderStyle)Enum.Parse(typeof(FormBorderStyle), m_SetAlarmBorder);
                    m_normDisp = false;
                    break;
            }
        }
        
        /// <Ver 1.15.1>アラーム発動
        private void InvokeAlarm()
        {
            if (!m_cancelAlarm)
            {
                m_alarm_mode = true;    //<Ver 1.14>アラーム発動フラグを立てる
                
                //<Ver 1.13>設定ファイルに保存するための現在の時計形状(前面か否か)を取得する
                if (this.WindowState == FormWindowState.Normal)
                    m_display_norm = true;
                else if (this.WindowState == FormWindowState.Minimized)
                    m_display_norm = false;

                // <Ver 1.10>アラーム発動時のフォーム形状を変更し点滅させる
                ChangeDisplay();
                FlashWnd();
            }
        }

        /// <Ver 1.00>ダブルクリックのメッセーハンドラ
        private void currentTime_DoubleClick(object sender, EventArgs e)
        {
            //<Ver 1.13>時計の位置を取得する getWinPos()
            //<Ver 1.15.1>値渡しで getTempWinPos() と共通化
            //<Ver 1.16.2>Pointクラスを使って書き換え
            GetFormPos(ref m_main_pos);
            //プログラムを終了する
            // <Ver1.14>最小化で終了した時にフォーム状態を設定ファイルに保存するために
            // <Ver 1.14> Application.Exit() 実行後 FormClosedイベントハンドラで処理する
            //Dispose(); // <Ver 1.14>Dispose()では FormClosedを捕らえられない
            Application.Exit(); 
        }

        // <Ver 1.00>DisplayのMouseDownイベントハンドラ
        // <Ver 1.10>マウスボタンが押されているときの処理
        private void currentTime_MouseDown(object sender, MouseEventArgs e)
        {
            // <Ver 1.10>左ボタンの場合は、マウス位置を取得しカーソルをHandに変更する
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //カーソルをHandに変更
                this.Cursor = Cursors.Hand;
                //位置を記憶する
                mousePoint = new Point(e.X, e.Y);
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }

            /// <Ver 1.10>右ボタンの場合は、時計を最小化する
            if ((e.Button & MouseButtons.Right) == MouseButtons.Right)
            {
                //<Ver 1.15>最初化前の時計の位置を*一時的に*保存 getTempWinPos()
                //<Ver 1.15.1> 参照渡しで getTempWinPos() と共通化
                //<Ver 1.16.2>Pointクラスを使って書き換え
                GetFormPos(ref m_temp_main_pos);
                //最小化する

                this.WindowState = FormWindowState.Minimized;
            }
        }

        /// <Ver 1.00>DisplayのMouseMoveイベントハンドラ
        /// <Ver 1.00>マウスが動いたときの処理
        private void currentTime_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //フォームの位置変更
                this.Left += e.X - mousePoint.X;
                this.Top += e.Y - mousePoint.Y;
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }
        }

        /// <Ver 1.10>キーを押した時のイベントハンドラ
        private void Display_KeyDown(object sender, KeyEventArgs e)
        {
#if DEBUG
            // アラーム動作確認用
            if (e.KeyCode == Keys.Tab)
            {
                InvokeAlarm();
            }
            //動作確認用ここまで
#endif
            if (e.KeyCode == Keys.Escape)
            {
                // <Ver 1.10>[Esc]キーを押した時にアラーム設定
                // <Ver 1.14.1>処理をまるごと setAlaem()にコピー
                //<Ver 1.16>アラーム発動中にアラーム設定画面を開かなくする
                if (!m_alarm_mode)
                {
                    if (m_allowAlarmDisp)   // <Ver 1.17>[外観の設定]起動時に[アラム設定]起動を許すフラグ
                        setAlarm();
                }
            }
            else if (e.KeyCode == Keys.F1)
            {
                // <Ver 1.16.9>[F1]から表示した時のみ[外観の設定]ボタンの表示フラグをture
                m_crtl_available = true;
                // <Ver 1.14.1>[F1]キーを押した時にAboutダイアログ表示
                showAbout();
            }
            else if (e.KeyCode == Keys.F12)
            {
                // <Ver 1.15.1>[F12]キーを押した時にアクティブフォームに張り付く
                // <Ver 1.17.3b>貼り付く設定の時は scClockを起動しない
                if (!m_stickDisp)
                    scClock();
            }
        }

        /// <Ver 1.15.1> scClockを起動する - 二重起動禁止
        private void scClock()
        {
            // 実行中のすべてのプロセスを取得する
            Process[] hProcesses = Process.GetProcesses();
            // 取得できたプロセスからプロセス名 "scClock" を探す
            bool existApp = false;
            foreach (Process hProcess in hProcesses)
            {
                if ("scClock" == hProcess.ProcessName.ToString())
                {
                    existApp = true;
                    break;
                }
            }

            if (existApp)
            {
                // <Ver 1.16.6>一旦、プロセスにある全ての scClock を解除
                // scClock のプロセスを取得
                Process[] ps = Process.GetProcessesByName("scClock");

                foreach (Process p in ps)
                {
                    //プロセスを強制的に終了させる
                    p.Kill();
                }
                try
                {
                    //改めて scClock を1つだけ起動
                    Process.Start("scClock");
                    //MessageBox.Show("scClock は既に起動しています.", About.getAssemblyProduct(),
                    //        MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (System.ComponentModel.Win32Exception)
                {
                    MessageBox.Show("scClock.exe が見つかりません.\n\nMyClock に同梱されている scClock.exe を同じフォルダに置いてください.", About.getAssemblyProduct(),
                            MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            else
            {
                //見つからない時のみ起動する
                try
                {
                    Process.Start("scClock");
                    //MessageBox.Show("scClock を起動しました.", About.getAssemblyProduct(),
                    //        MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (System.ComponentModel.Win32Exception)
                {
                    MessageBox.Show("scClock.exe が見つかりません.\n\nMyClock に同梱されている scClock.exe を同じフォルダに置いてください.", About.getAssemblyProduct(),
                            MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
        }

        /// <Ver 1.14.1>アラームを設定する
        private void setAlarm()
        {
            // <Ver 1.14>前面表示時のアラーム画面サイズフラグの更新
            Alarm.m_Norm_alarm = m_Norm_alarm;
            // <Ver 1.14>アイコン表示時のアラーム画面サイズフラグの更新
            Alarm.m_Icon_alarm = m_Icon_alarm;
            // <Ver 1.11>アラーム解除フラグの更新
            Alarm.m_alarmCanceled = m_cancelAlarm;
            // <Ver 1.10>m_Alarmが値を格納している時は、Alatm.m_Alarmの値を更新する
            if (m_Alarm != null)
                Alarm.m_alarmTime = m_Alarm;

            // <Ver 1.10>アラーム設定フォームのインスタンス作成
            Alarm dlg = new Alarm();
            // <Ver 1.10>アラーム設定フォームをモーダルダイアログで表示
            var result = dlg.ShowDialog(this);
            if (result == DialogResult.OK)
            {
                // OK が返ってきたときの処理
                // <Version 1.13>アラーム画面設定
                m_Norm_alarm = Alarm.m_Norm_alarm;
                m_Icon_alarm = Alarm.m_Icon_alarm;
                // <Ver1.11>アラーム解除フラグ
                m_cancelAlarm = Alarm.m_alarmCanceled;
                // <Ver 1.10>アラーム設定時刻
                string m_time = Alarm.m_alarmTime;
                if (m_time != null || m_time != m_Alarm)
                {
                    // アラーム設定時刻が変更された場合、static m_Alarm 変数を更新
                    m_Alarm = Alarm.m_alarmTime;
                }
            }
            else if (result == DialogResult.Yes)
            {
                m_Norm_alarm = Alarm.m_Norm_alarm;
                m_Icon_alarm = Alarm.m_Icon_alarm;
                m_cancelAlarm = Alarm.m_alarmCanceled;
                string m_time = Alarm.m_alarmTime;
                if (m_time != null || m_time != m_Alarm)
                {
                    // アラーム設定時刻が変更された場合、static m_Alarm 変数を更新
                    m_Alarm = Alarm.m_alarmTime;
                }
                InvokeAlarm();
            }

            // MSDNでダイアログの破棄をすべきと推奨している
            dlg.Dispose();
        }

        ///<Ver 1.16.2>Aboutダイアログの表示
        private void showAbout()
        {
            About.m_show_about = m_show_about;
            // <Ver 1.16.2>Aboutをモーダルダイアログで表示
            About dlg = new About();
            var result = dlg.ShowDialog(this);
            if (result == DialogResult.OK)
            {
                m_show_about = About.m_show_about;
            }
            else if (result == DialogResult.Yes)
            {
                // <Ver 1.16.9>外観変更機能呼び出し
                setCtrl();
            }

            // MSDNでダイアログの破棄をすべきと推奨している
            dlg.Dispose();
        }

        /// <Ver 1.10>ラベルをシングルクリックした時のイベントハンドラ
        /// <Ver 1.14>フォーム形状を元に戻すのは、アラーム発動フラグが下りている時のみに制限
        private void currentTime_Click(object sender, EventArgs e)
        {
            // <Ver 1.14>フォームを元に戻すのは、
            // <Ver 1.14>アラーム発動フラグが立っている時のみに変更(機能追加のため)
            if (m_alarm_mode == true)
            {
                // <Ver 1.10>フォーム形状を元に戻し、点滅を止める
                StopFlashWnd();
                ResumeDisplay();
                // <Ver 1.14>アラーム発動フラグを下ろす
                m_alarm_mode = false;
                // <Ver 1.16.9>アラーム発動からの復帰フラグを立てる
                m_resumedDisp = true;
            }
        }


        private void Display_Click(object sender, EventArgs e)
        {
            // <Ver 1.14>フォームを元に戻すのは、
            // <Ver 1.14>アラーム発動フラグが立っている時のみに変更(機能追加のため)
            if (m_alarm_mode == true)
            {
                // <Ver 1.10>フォーム形状を元に戻し、点滅を止める
                StopFlashWnd();
                ResumeDisplay();
                // <Ver 1.14>アラーム発動フラグを下ろす
                m_alarm_mode = false;
                // <Ver 1.16.9>アラーム発動からの復帰フラグを立てる
                m_resumedDisp = true;
            }
        }

        /// <Ver 1.10>時計の見え方を変える
        /// <Ver 1.14>アラーム発動時の時計の形状を選択できるようにしたのでロジック変更
        private void ChangeDisplay()
        {
            // <Ver 1.17.3b>貼り付く設定の時はフラグを解除
            if (m_temp_stickDisp = m_stickDisp)
                m_stickDisp = false;

            // <Ver 1.15>時計があるディスプレイの高さと幅を取得
            int screenH = System.Windows.Forms.Screen.GetBounds(this).Height;
            int screenW = System.Windows.Forms.Screen.GetBounds(this).Width;

            // ウィンドウスタイルと位置を変更する
            this.FormBorderStyle = FormBorderStyle.FixedToolWindow;
            this.ClientSize = m_DispNormSize; // <Ver 1.17.3> すぐに設定サイズに戻すために追加
            this.Text = "It's Time! - Click Here";
            if (m_display_norm)
            {
                // <Ver 1.15>アラーム発動直前の時計の位置を*一時的に*保存
                //<Ver 1.16.2>Pointクラスを使って書き換え
                GetFormPos(ref m_temp_main_pos);
                switch (m_Norm_alarm)
                {
                    case 1:
                        this.WindowState = FormWindowState.Normal;
                        this.Opacity = 1D;
                        break;
                    case 2:
                        this.WindowState = FormWindowState.Maximized;
                        this.Opacity = 0.5D;
                        break;
                    case 3:
                        this.WindowState = FormWindowState.Normal;
                        this.Opacity = 1D;
                        this.Location = new Point((screenW - this.Width) / 2, (screenH - this.Height) / 2);
                        break;
                }
            }
            else
            {
                switch (m_Icon_alarm)
                {
                    case 1:
                        this.WindowState = FormWindowState.Normal;
                        this.Opacity = 1D;
                        break;
                    case 2:
                        this.WindowState = FormWindowState.Maximized;
                        this.Opacity = 0.5D;
                        break;
                    case 3:
                        this.WindowState = FormWindowState.Normal;
                        this.Opacity = 1D;
                        this.Location = new Point((screenW - this.Width) / 2, (screenH - this.Height) / 2);
                        break;
                }
            }
        }

        /// <Ver 1.10>時計の見え方を元に戻す
        private void ResumeDisplay()
        {
            // <Ver 1.17.3b>貼り付く設定の時はフラグを解除
            m_stickDisp = m_temp_stickDisp;

            this.WindowState = FormWindowState.Normal;
            this.FormBorderStyle = FormBorderStyle.None;
            this.Opacity = 0.5D;
            this.Text = "MyClock";
            // <Ver 1.15>元が前面表示のとき
            if (m_display_norm)
            {
                // <Ver 1.15>アラームで画面中央表示のとき、元の位置に戻す
                if (m_Norm_alarm == 3)
                    //<Ver 1.16.2>Pointクラスを使って書き換え
                    this.Location = m_temp_main_pos;
            }
            else
            {
                if (m_Icon_alarm == 3)
                {
                    //<Ver 1.16.2>Pointクラスを使って書き換え
                    this.Location = m_temp_main_pos;
                }
                //<Ver 1.14>復元時はノーマルスタイルが使いやすそうなので以下はコメントアウト
                //    this.WindowState = FormWindowState.Minimized;
            }
        }

        // <Ver 1.10>ウィンドウを点滅させる
        private void FlashWnd()
        {
            FLASHWINFO flsh = new FLASHWINFO();
            flsh.cbSize = Convert.ToUInt32(Marshal.SizeOf(flsh));
            flsh.hwnd = this.Handle;
            flsh.dwFlags = WinAPI.FLASHW_ALL | WinAPI.FLASHW_TIMER;
            //flsh.uCount = 5;
            flsh.dwTimeout = 100;

            WinAPI.FlashWindowEx(ref flsh);
        }

        /// <Ver 1.10>ウィンドウの点滅を止める
        private void StopFlashWnd()
        {
            FLASHWINFO flsh = new FLASHWINFO();
            flsh.cbSize = Convert.ToUInt32(Marshal.SizeOf(flsh));
            flsh.hwnd = this.Handle;
            flsh.dwFlags = WinAPI.FLASHW_STOP;

            WinAPI.FlashWindowEx(ref flsh);
        }

        /// <Ver 1.16.9>Displayオブジェクトを保持するためのフィールド
        private static Display _dispInstance;
        /// <Ver 1.16.9>Displayオブジェクトを取得、設定するためのプロパティ
        public static Display DispInstance
        {
            get { return _dispInstance; }
            set { _dispInstance = value; }
        }

        /// <Ver 1.16.9>currentTime.BackColorを取得、設定するためのプロパティ
        public Color DispBackColor
        {
            get { return currentTime.BackColor; }
            set { currentTime.BackColor = value; }
        }

        /// <Ver 1.16.9>currentTime.Fontを取得、設定するためのプロパティ
        public Font DispFont
        {
            get { return currentTime.Font; }
            set { currentTime.Font = value; }
        }

        /// <Ver 1.16.9>currentTime.FontColorを取得、設定するためのプロパティ
        public Color DispForeColor
        {
            get { return currentTime.ForeColor; }
            set { currentTime.ForeColor = value; }
        }

        /// <Ver 1.16.9>Display.Sizeを取得、設定するためのプロパティ
        public Size DispSize
        {
            get { return this.ClientSize; }
            set { this.ClientSize = value; }
        }

        /// <Ver 1.16.9>Display.Opacityを取得、設定するためのプロパティ
        public double DispOpacity
        {
            get { return this.Opacity; }
            set { this.Opacity = value; }
        }

        /// <Ver 1.16.9>currentTime.BorderStyleを取得、設定するためのプロパティ
        public BorderStyle DispBorder
        {
            get { return currentTime.BorderStyle; }
            set { currentTime.BorderStyle = value; }
        }

        /// <Ver 1.16.9>[外観の変更]画面を表示する
        private void setCtrl()
        {
            Ctrl dlg = new Ctrl();
            dlg.Show(this);
        }

        /// <Ver 1.17.3b>timer_Stick.Enabledを取得、設定するためのプロパティ
        public void DispStickTimer(bool flag)
        {
            timer_Stick.Enabled = flag;
        }

        /// <Ver 1.13>終了時にアプリ設定を保存する (MyClock.exe.config)
        /// <Ver 1.14>設定保存を FormClosed()で行うように変更した
        /// <Ver 1.15>起動時メッセージ設定を保存する
        /// <Ver 1.16.7> 設定ファイル入出力は、クラス Config にまとめた
        private void Display_FormClosed(object sender, FormClosedEventArgs e)
        {
            //<Ver1.13>時計サイズの保存を行うため、DoubleClickイベントハンドラから移動
            // <Ver 1.16.7> 設定ファイル入出力は、クラス Config にまとめた
            Config Conf = new Config();
            Conf.SetConfig(); // Ver 1.17.3d 以降は MyClock.execonfig に保存しない
            Conf.SetIniFile();  // Ver 1.17.3d 以降は INI ファイルを設定ファイルとする
        }

        /// <Ver 1.17.3b>定期的にアクティブウィンドウを取得する
        private void timer_Stick_Tick(object sender, EventArgs e)
        {
            if (m_stickDisp)
            {
                GetActiveWindow();
            }
        }

        /// <Ver 1.17.3b>アクティブなウィンドウを取得する
        private void GetActiveWindow()
        {
            // アクティブウィンドウのハンドルを取得する
            IntPtr hWnd = WinAPI.GetForegroundWindow();
            // アクティブウィンドウの位置を取得
            RECT winRect = new RECT();
            WinAPI.GetWindowRect(hWnd, ref winRect);
            int id;
            WinAPI.GetWindowThreadProcessId(hWnd, out id);
            string pName = Process.GetProcessById(id).ProcessName;

            // scClock自身がアクティブでない時 / MyClock がアクティブでない時
            if (Form.ActiveForm != this)
            {
                if (pName != "MyClock")
                {
#if DEBUG
                    if (pName != "MyClock.vshost")
#endif
                        // scClockをアクティブウィンドウへ移動する
                        MoveDisplayTo(winRect);
                }
            }
        }

        // <Ver 1.17.3b>キャプションバーボタンの寸法取得
        private static int buttonW = SystemInformation.CaptionButtonSize.Width;
        private static int buttonH = SystemInformation.CaptionButtonSize.Height;
        // <Ver 1.17.3b>ウィンドウ枠の幅を取得
        private static int winBorder = SystemInformation.Border3DSize.Width;

        /// <Ver 1.17.3b>scClockをアクティブウィンドウへ移動する
        private void MoveDisplayTo(RECT rect)
        {
            // 表示位置(X, Y)の取得
            int X = rect.right - winBorder - (buttonW * 4) - this.Width;
            int Y = rect.top; // キャプションバーの上端 (scClockと異なる)

            // 幅が小さすぎる時はウィンドウの上に移動する
            if (X < rect.left + buttonW)
            {
                X = rect.left + buttonW;
                Y = rect.top - this.Height;
            }
            // <Ver 1.17.3c>スクリーン上端以下に表示する
            if (Y < 0) Y = 0; 
            // 時計を移動
            this.Left = X;
            this.Top = Y;
        }
    }
}
